/*
 * Interactive demonstration of Catmull-Rom splines
 * based on Processing "curve" function or use
 * of "curveVertex" within a shape.
 *
 * The use may press space to pick new random vertices.
 *
 * The user may mouse drag existing point to new location.
 *
 * Sketch by Michael Goldwasser
 */


// pardon our advance use of arrays.
// (explanation coming soon in lecture)
int N = 4; 
float x[] = new float[N];
float y[] = new float[N];
int weight = 5;
int active = -1;
int fontSize = 30;
int tangentLength = 20;
boolean showSkeleton = true;

void setup() {
  size(800, 600);
  textSize(fontSize);
  textAlign(CENTER, CENTER);
  randomize();
}

void draw() {
  background(255);

  // first draw the actual curve
  stroke(127);
  strokeWeight(weight);
  fill(240, 127, 71);
  bezier(x[0], y[0], x[1], y[1], x[2], y[2], x[3], y[3]);

  // then draw the skeleton that defines the curve
  if (showSkeleton) {
    strokeWeight(0.5*weight);
    stroke(0, 0, 255, 127);
    line(x[0], y[0], x[1], y[1]);
    line(x[1], y[1], x[2], y[2]);
    line(x[2], y[2], x[3], y[3]);
    // compute midpoints
    float xa = 0.5*(x[0]+x[1]);
    float ya = 0.5*(y[0]+y[1]);
    float xb = 0.5*(x[1]+x[2]);
    float yb = 0.5*(y[1]+y[2]);
    float xc = 0.5*(x[2]+x[3]);
    float yc = 0.5*(y[2]+y[3]);
    stroke(255, 0, 0, 127);
    line(xa, ya, xb, yb);
    line(xb, yb, xc, yc);
    float xd = 0.5*(xa+xb);
    float yd = 0.5*(ya+yb);
    float xe = 0.5*(xb+xc);
    float ye = 0.5*(yb+yc);
    stroke(0, 255, 0, 127);
    line(xd, yd, xe, ye);
  }

  // now show vertices on top
  fill(0);
  for (int j=0; j < N; j++) {
    text(""+(j+1), x[j], y[j]);
  }
}

void randomize() {
  for (int j=0; j < N; j++) {
    x[j] = random(0.2*width, 0.8*width);
    y[j] = random(0.2*height, 0.8*height);
  }
}

// when mouse is pressed, see if near an existing point
void mousePressed() {
  for (int j=0; j < N; j++) {
    float distSquared = pow(mouseX-x[j], 2) + pow(mouseY-y[j], 2);
    if (distSquared < 0.25*fontSize*fontSize) {
      active = j;
      break;
    }
  }
}

void mouseDragged() {
  if (active != -1) {
    x[active] = mouseX;
    y[active] = mouseY;
  }
}

void mouseReleased() {
  active = -1;
}

void keyPressed() {
  if (key == ' ') {
    showSkeleton = !showSkeleton;
  } else if (key == 'r') {
    randomize();
  }
}

