/*
 * Interactive demonstration of Catmull-Rom splines
 * based on Processing "curve" function or use
 * of "curveVertex" within a shape.
 *
 * The use may press space to pick new random vertices.
 *
 * The user may mouse drag existing point to new location.
 *
 * Sketch by Michael Goldwasser
 */


// pardon our advance use of arrays.
// (explanation coming soon in lecture)
int N = 6; 
float x[] = new float[N];
float y[] = new float[N];
int weight = 5;
int active = -1;
int fontSize = 30;
int tangentLength = 20;

void setup() {
  size(800,600);
  textSize(fontSize);
  textAlign(CENTER,CENTER);
  randomize();
}

void draw() {
  background(255);
  
  // first draw the curve
  stroke(127);
  strokeWeight(weight);
  fill(240, 127, 71);
  beginShape();
  curveVertex(x[0],y[0]);  // first point is also control
  for (int j=0; j < N; j++) {
    curveVertex(x[j], y[j]);
  }
  curveVertex(x[N-1],y[N-1]);  // last point is also control
  endShape();
  
  // now show vertices on top
  fill(0);
  for (int j=0; j < N; j++) {
    text(""+(j+1), x[j], y[j]);
  }
}

void randomize() {
  for (int j=0; j < N; j++) {
    x[j] = random(0.2*width, 0.8*width);
    y[j] = random(0.2*height, 0.8*height);
  }
}

// when mouse is pressed, see if near an existing point
void mousePressed() {
  for (int j=0; j < N; j++) {
    float distSquared = pow(mouseX-x[j],2) + pow(mouseY-y[j],2);
    if (distSquared < 0.25*fontSize*fontSize) {
      active = j;
      break;
    } 
  }
}

void mouseDragged() {
  if (active != -1) {
    x[active] = mouseX;
    y[active] = mouseY;
  }
}

void mouseReleased() {
  active = -1;
}

void keyPressed() {
  if (key == 'r') {
    randomize();
  }
}

